#!/usr/bin/perl -w
use strict;
#use diagnostics;

#(This license is from zlib of Jean-loup Gailly and Mark Adler)
#
# Copyright (C) 2002 Frederick Dean <software@fdd.com>
#
#  This software is provided 'as-is', without any express or implied
#  warranty.  In no event will the authors be held liable for any damages
#  arising from the use of this software.
#
#  Permission is granted to anyone to use this software for any purpose,
#  including commercial applications, and to alter it and redistribute it
#  freely, subject to the following restrictions:
#
#  1. The origin of this software must not be misrepresented; you must not
#     claim that you wrote the original software. If you use this software
#     in a product, an acknowledgment in the product documentation would be
#     appreciated but is not required.
#  2. Altered source versions must be plainly marked as such, and must not be
#     misrepresented as being the original software.
#  3. This notice may not be removed or altered from any source distribution.
#
# Contributor(s):    Frederick Dean <software@fdd.com>

require cgi;
require db;
require cvsdb;
require cvsweb;

package branch_html;


sub unixTimestamp {
   return ::time2str($config::timeformat,$_[0]);
}
sub readableDuration {
   return cvsweb::readableTime($db::now - $_[0],1);
}
sub logComment {
   return common_html::bugzillifyHtml(cgi::htmlQuote($_[0]));
}
sub hexdump {
   return unpack("H*",$_[0]);
}
sub revision {
   my($value,$filenameWithPath,undef) = @_;
   return cgi::Link("$filenameWithPath---checkout.html","rev=$value") . cgi::htmlQuote($value) . "</a>";
}
sub branchname {
   my($branchname,$filenameWithPath,undef) = @_;
   return cgi::Link($filenameWithPath,"branch=$branchname") . cgi::htmlQuote($branchname) . "</a>\n";
}

my(%colsql,%coltitle,%coladmin,%colfunc,%coldesc,$allcols);
sub DefCol {
   my($nickname,$title,$sql,$colfunc,$descript,$urlparam) = @_;
   $coltitle{$nickname} = $title;  # top of table
   $colsql{$nickname} = $sql;
   $colfunc{$nickname} = $colfunc;
   $coldesc{$nickname} = $descript; # for column selection
   $allcols .= ".$nickname";
}
#       nick       title             sql                    type                descript
DefCol("name",     "name",          "btime.branchname",    \&branchname,       "Branch name");
DefCol("num",      "num",           "btime.branchnum",     undef,              "Branch num");
DefCol("hexacl",   "hex of ACL",    "btime.acl",           \&hexdump,          "Hex of Access Control List") if $config::debug;
DefCol("owner",    "branch owner",  "user.username",       undef,              "Branch owner");
#DefCol("fname",    "fname",         "dirent.fname",        undef,              "Filename");
#DefCol("file_id",  "file_id",       "dirent.file_id",      undef,              "database file id");
DefCol("rev",      "rev",           "revision.revision",   \&revision,         "Recent revision (on this branch)");
DefCol("date",     "date",          "revision.timestamp",  \&unixTimestamp,    "When was it committed" );
DefCol("age",      "age",           "revision.timestamp",  \&readableDuration, "Age of recent revision");
DefCol("length",   "length",        "revision.length",     undef,              "File length of displayed revision");
DefCol("author",   "rev author",    "comment.author",      undef,              "Author of recent revision");
DefCol("log",      "last log entry","comment.descr",       \&logComment,       "Full log entry of recent revision");
DefCol("comment_id","comment_id",   "revision.comment_id", undef,              "Log entry of recent revision");
my $default_cols = "name.num.rev.age.author.log.owner";

sub branch_html
{
   my($filename,$file_id,$inforef) = @_;

   common_html::content_type("text/html");
   my $cols = $cgi::form{'cols'} || $default_cols;
   my @cols = split(/[^\w\-]+/,$cols); # we don't trust this list of cols
   my @cols2 = ();  # this is were we store the list of cols after we checked they are okay
   my($sql,$heading);
   my $sort = $cgi::form{'sort'} || "-age";
   my $desc = ($sort =~ s/^-//) ? "DESC" : "";  # remove leading negation
   $sort = "age" unless(defined $colsql{$sort});
   for my $nick (@cols) {  # for each column we are preparing for
      next if(!defined($colsql{$nick}));
      push(@cols2,$nick);
      $sql .= ",$colsql{$nick}";
      my $color = "000080";
      $color = "05a555" if $sort eq $nick;
      my $direction = "";
      $direction = "-" if $sort eq $nick && ! $desc;
      $heading .= "<th bgcolor=$color>".cgi::Link("$filename---branch.html","sort=$direction$nick").
                  "<font color=ffffff>$coltitle{$nick}</font></a></th>\n";
   }
   $heading = "<table width=100%><tr>\n$heading</tr>\n";
   # First assume $file_id is a file (not a directory).
   my $fromwhere = "FROM btime, revision WHERE btime.file_id = $file_id AND revision.file_id = $file_id ";
   if($$inforef{'type'} eq "dir") {  # if $file_id is a directory
      $fromwhere = "FROM btime, revision, dirent ".
                   "WHERE dirent.parent = $file_id AND btime.file_id = dirent.file_id AND revision.file_id = dirent.file_id ";
   };
   db::SendSQL("CREATE TEMPORARY TABLE latest3 ".
               "SELECT branchname, MAX(revision.rev_id) AS rev_id ".
               $fromwhere .
               "AND revision.branchnum = btime.branchnum ".
               "AND (revision.timestamp <= btime.timestamp OR btime.timestamp IS NULL) ".
               "GROUP BY branchname");
=junk
   db::SendSQL("CREATE TEMPORARY TABLE early_commit ".    # earliest commit
               "SELECT branchname, MIN(revision.rev_id) AS rev_id ".
               $fromwhere .
               "AND revision.branchnum = btime.branchnum ".
               "AND btime.timestamp IS NULL ".
               "GROUP BY branchname");
=cut
   db::getDbNowTime();
   # btime should be a left join below, but MySQL took forever when that happens
   db::SendSQL("SELECT btime.acl, btime.owner $sql ".
               "FROM latest3 ".
               "LEFT JOIN revision ON latest3.rev_id = revision.rev_id ".
               "LEFT JOIN btime ON btime.file_id = revision.file_id AND btime.branchname = latest3.branchname AND btime.timestamp IS NULL ".
               "LEFT JOIN comment ON revision.comment_id = comment.comment_id " .
               "LEFT JOIN user ON btime.owner = user.userid ".
               "ORDER BY $colsql{$sort} $desc" );
   my $rowIndex = 0;
   my $table = $heading;
   while(my @row = db::FetchSQLData()) {  # for each row
      $table .= ' <tr bgcolor="#' . (($rowIndex & 1) ? "FFFFFF" : "dFdFFF") . "\">\n";
      my($acl,$owner) = splice(@row,0,2,());
      for my $nick (@cols2) {  # for every column of the row
         my $value = shift(@row) || "";
         $table .= "  <td>";
         if(length $value == 0 && $nick ne "name") {
            $table .= "&nbsp;"; # netscape requires at least one space for background color
         } elsif(defined $colfunc{$nick}) {
            $table .= &{$colfunc{$nick}}($value,"$filename");
         } else {
            $table .= cgi::htmlQuote($value);
         }
         $table .= "</td>\n";
      }
      $table .= " </tr>\n";
      $table .= "</table>\n$heading" if (($rowIndex++)&127) == 127;  # huge tables display slowly
   }
   $table .= "</table>\n";
   db::SendSQL("DROP TABLE latest3");
   ###db::SendSQL("DROP TABLE early_commit");
   $cgi::keyword{'branchTable'} = $table;
   $cgi::keyword{'numBranches'} = $rowIndex;
   $cgi::keyword{'allColsLink'} = cgi::Link("$filename---branch.html","cols=$allcols");
   print cgi::expand_template_keywords("template/branch-file-template.html");
}


1;

