#!/usr/bin/perl -w -T
use strict;
#use diagnostics;

#(This license is from zlib of Jean-loup Gailly and Mark Adler)
#
# Copyright (C) 2002 Frederick Dean <software@fdd.com>
#
#  This software is provided 'as-is', without any express or implied
#  warranty.  In no event will the authors be held liable for any damages
#  arising from the use of this software.
#
#  Permission is granted to anyone to use this software for any purpose,
#  including commercial applications, and to alter it and redistribute it
#  freely, subject to the following restrictions:
#
#  1. The origin of this software must not be misrepresented; you must not
#     claim that you wrote the original software. If you use this software
#     in a product, an acknowledgment in the product documentation would be
#     appreciated but is not required.
#  2. Altered source versions must be plainly marked as such, and must not be
#     misrepresented as being the original software.
#  3. This notice may not be removed or altered from any source distribution.
#
# Contributor(s):    Frederick Dean <software@fdd.com>

BEGIN { unshift @INC, "/var/www/html/dcvs" };

$ENV{'PATH'} = '/bin:/usr/bin';
delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};

require config;
print STDERR "\n\n" if $config::debug || $config::debug;  # useful for debugging
require db;
require cvsdb;
require common_html;
use Date::Format;          
use Date::Parse;        
require cgi;
require session;

#print("content-type: text/html\n\n"); 
cgi::MakeFormHash();
cgi::MakeCookieHash();
$cgi::keyword{'warning'} = "";
my $path = cgi::GetPathAfterCgi();

$path =~ s/---(\w+)\.html$//;
my $command = $1 || "";
$path = cvsdb::cleanPath($path);
$path = "/" if $path eq "";
$cgi::keyword{'pwd'} = $path;

$path =~ s/\.diff$// if $cgi::form{'r2'};   # remove optional diff suffix
my $file_id = cvsdb::id($path);
$cgi::keyword{'file_id'} = $file_id;  # only for debugging

my $inforef =  cvsdb::admin_info($file_id);
common_html::linkedPath_keyword($path,$inforef);
common_html::url_keywords($path);
session::check();
if($file_id == -1) {
   require error_html;
   error_html::notfound_html();
   exit;
}

#session::require_login();

# We do our command with a dummy file here so people don't save with wrong filename.
# No CVS repository can have a filename or directory named CVS, so this is non-conflicting part of namespace.
if($command) {
   if($command eq "ls") {
      require ls_html;
      ls_html::ls_html($path,$file_id,$inforef); 
   } elsif($command eq "log") {
      require log_html;
      log_html::log_html($path,$file_id,$inforef); 
   } elsif($command eq "checkout" || $command eq "co") {
      require co_html;
      co_html::checkout_html($path,$file_id,$inforef); 
   } elsif($command eq "branch") {
      require branch_html;
      branch_html::branch_html($path,$file_id,$inforef); 
   } elsif($command eq "users") {
      require users_html;
      users_html::users_html($path,$file_id,$inforef); 
   } elsif($command eq "user") {
      require user_html;
      user_html::user_html($path,$file_id,$inforef); 
   } elsif($command eq "access") {
      require access_html;
      access_html::access_html($path,$file_id,$inforef); 
   } elsif($command eq "prefs") {
      require prefs_html;
      prefs_html::prefs_html($path,$file_id,$inforef); 
   } else {
      require error_html;
      error_html::fatal("Command \"$command\" not found.\n");
   };
} elsif($cgi::form{'r2'}) {
   require diff_html;
   diff_html::diff_html($path,$file_id,$inforef);
} elsif($cgi::form{'rev'}) {  
   require co_html;
   co_html::checkout_html($path,$file_id,$inforef);
} elsif($cgi::form{'tag'}) {
   require co_html;
   co_html::checkout_html($path,$file_id,$inforef);
} else {
   require ls_html;
   ls_html::ls_html($path,$file_id,$inforef); 
};

if($config::debug) {
   cgi::PrintFormHash();
   session::PrintGroupHash();
   cgi::PrintEnv();
   cgi::PrintCookieHash();
   cgi::PrintKeywordHash();
   print("<hr><font size=+2>$0 done</font>\n");
}

