#!/usr/bin/perl -w
use strict;
#use diagnostics;

#(This license is from zlib of Jean-loup Gailly and Mark Adler)
#
# Copyright (C) 2002 Frederick Dean <software@fdd.com>
#
#  This software is provided 'as-is', without any express or implied
#  warranty.  In no event will the authors be held liable for any damages
#  arising from the use of this software.
#
#  Permission is granted to anyone to use this software for any purpose,
#  including commercial applications, and to alter it and redistribute it
#  freely, subject to the following restrictions:
#
#  1. The origin of this software must not be misrepresented; you must not
#     claim that you wrote the original software. If you use this software
#     in a product, an acknowledgment in the product documentation would be
#     appreciated but is not required.
#  2. Altered source versions must be plainly marked as such, and must not be
#     misrepresented as being the original software.
#  3. This notice may not be removed or altered from any source distribution.
#
# Contributor(s):    Frederick Dean <software@fdd.com>

require cgi;
require db;
require cvsdb;
require cvsweb;

package log_html;

sub SingleCommitBlurb
{
   my($filelistref,$inforef) = @_;
   my($comment,$timestamp,$author,$realname) = @$inforef[5..8];
   my $blurb = "<hr noshade>\n";
   $blurb .= "comitted <i>".::time2str($config::timeformat,$timestamp)."</i>\n";
   $blurb .= " (".cvsweb::readableTime($db::now - $timestamp,1)." ago)\n";
   $blurb .= " <i>by $author</i><br>\n" if $author;
   $blurb .= "<table>\n";
   for my $fileinforef (@$filelistref) {
      my($fname,$path,$rev,$branch) = @$fileinforef;
      $branch = "unknown" if(!defined($branch));
      $blurb .= " <tr><td>$path/".cgi::Link("$path/$fname")."$fname</a>&nbsp;</td>\n";
      $blurb .= " <td>".cgi::Link("$path/$fname","rev=$rev")."$rev</a></td>\n";
      $blurb .= " <td>(".cgi::Link("$path/$fname")."download</a>)</td>\n";
      my $prev = cvsdb::GuessPreviousRev($rev);
      $blurb .= " <td>".cgi::Link("$path/$fname.diff","r1=$rev","r2=$prev")."diff-to-previous</a></td>\n" unless $rev eq "1.1";
      $blurb .= " <td> -- branch ".cgi::htmlQuote($branch)."</td>\n" if($branch ne "MAIN");
      $blurb .= " </tr>\n";
   };
   $blurb .= "</table>\n";
   $comment = common_html::bugzillifyHtml(cgi::htmlQuoteForPre($comment));
   $blurb .= "<pre>$comment</pre>\n\n";
   return $blurb;
}

sub log_html
{
   my($path,$file_id,$inforef) = @_;
 
   common_html::content_type("text/html");
   if($$inforef{'type'} ne "dir") {
      require ls_html;
      ls_html::ls_html($path,$file_id,$inforef);
      return;
   }
   my($andwhere,$parentspec1,$parentspec2) = ("","","");
   if($file_id) {  # if not the root directory
      $parentspec1 = "descendent, ";
      $parentspec2 = "AND descendent.parent = $file_id AND descendent.child = dirent.parent ";
   }
   $andwhere .=  " AND comment.author = " . db::SqlQuote($cgi::form{'author'}) if($cgi::form{'author'});
   my($after,$before) = ($cgi::form{'after'},$cgi::form{'before'});
   if(! defined $before && ! defined $after) {
      $after = time() - 7*24*3600;
      $cgi::form{'after'} = ::time2str("%b %e",$db::now - 7*24*3600);
   };
   if($after) {
      $after = ::str2time($after) if($after !~ /^\d+/);
      if($after) {
         $andwhere .=  " AND comment.timestamp > $after";
      } else {
         print("<font color=red size=+1>Could not understand \"after ".cgi::htmlQuote($cgi::form{'after'})."\"</font><br>\n");
      };
   }
   if($before) {
      $before = ::str2time($before) if($before !~ /^\d+/);
      if($before) {
         $andwhere .=  " AND comment.timestamp < $before";
      } else {
         print("<font color=red size=+1>Could not understand \"before ".cgi::htmlQuote($cgi::form{'before'})."\"</font><br>\n");
      };
   }
   $cgi::keyword{'after'} = cgi::htmlQuote($cgi::form{'after'}) || "";
   $cgi::keyword{'before'} = cgi::htmlQuote($cgi::form{'before'}) || "";
   common_html::branchesWidget_keyword("dirent.parent = $file_id"); 
   db::getDbNowTime();
   db::SendSQL("SELECT comment.comment_id, dirent.fname, dirent2.path, revision.revision, btime.branchname, ".
            "comment.descr, comment.timestamp, comment.author, user.realname ".
            "FROM comment, $parentspec1 revision, dirent ".
            "LEFT JOIN btime ON btime.file_id = revision.file_id AND btime.branchnum = revision.branchnum ".
            "AND btime.timestamp IS NULL ". 
            "LEFT JOIN user ON comment.author = user.username ".
            "LEFT JOIN dirent AS dirent2 ON dirent.parent = dirent2.file_id ".
            "WHERE dirent.file_id = revision.file_id AND revision.comment_id = comment.comment_id $parentspec2 $andwhere ".
            "ORDER BY -comment.timestamp, descr, path, fname"); 
   my @filelist;  # contains references to small lists of (filename, path, revision, branch)
   my $previnforef;
   my $log = "";  # here we build the big html log of multiple commits
   my $numCommits = 0;
   # Because a record comes back for each file, several records can refer to the same commit.
   # We want to display all the files of one commit together so wait until the comment changes before we print a blurb.
   while(my @commitinfo = db::FetchSQLData()) {
      $numCommits++;
      if(defined $previnforef && $commitinfo[0] != $$previnforef[0]) {  # if the comment_id has changed
         $log .= SingleCommitBlurb(\@filelist,$previnforef);  # print a blurb for the previous comment_id
         @filelist = ();  # empty the file list
      };
      $previnforef = \@commitinfo;  # got to keep a whole row for the log message text, author, etc.
      push(@filelist,[@commitinfo[1..4]]);
   };
   $log .= SingleCommitBlurb(\@filelist,$previnforef) if defined $previnforef;
   $cgi::keyword{'numCommits'} = $numCommits;
   $cgi::keyword{'log'} = $log;
   print cgi::expand_template_keywords("template/log-template.html");
}

1;

