#!/usr/bin/perl -w
use strict;
#use diagnostics;

#(This license is from zlib of Jean-loup Gailly and Mark Adler)
#
# Copyright (C) 2002 Frederick Dean <software@fdd.com>
#
#  This software is provided 'as-is', without any express or implied
#  warranty.  In no event will the authors be held liable for any damages
#  arising from the use of this software.
#
#  Permission is granted to anyone to use this software for any purpose,
#  including commercial applications, and to alter it and redistribute it
#  freely, subject to the following restrictions:
#
#  1. The origin of this software must not be misrepresented; you must not
#     claim that you wrote the original software. If you use this software
#     in a product, an acknowledgment in the product documentation would be
#     appreciated but is not required.
#  2. Altered source versions must be plainly marked as such, and must not be
#     misrepresented as being the original software.
#  3. This notice may not be removed or altered from any source distribution.
#
# Contributor(s):    Frederick Dean <software@fdd.com>

require cgi;
require db;
require cvsweb;
require cvsdb;

package ls_html;

sub htmlLsForFile 
{
   my($filename,$file_id,$inforef) = @_;
   if($filename =~ /\/$/) {
      $cgi::keyword{'warning'} .= "<font size=+2 color=red>File ".cgi::htmlQuote($filename)." doesn't exist</font>";
      return;
   }; 
   my %revision;
   cvsdb::GetRevInfoFromDb(\%revision,$file_id,$cgi::form{'branch'});
   $cgi::keyword{'revisionList'} = common_html::HtmlRevInfo($filename,$file_id,\%revision);
   #(cgi::keyword{'numRevs'},cgi::keyword{'revisionList'}) = common_html::HtmlRevInfo($filename,$file_id,\%revision);
   common_html::branchesWidget_keyword("dirent.file_id = $file_id"); 
   print cgi::expand_template_keywords("template/ls-file-template.html");
} 

sub unixTimestamp {
   return ::time2str($config::timeformat,$_[0]);
}
sub readableDuration {
   return cvsweb::readableTime($db::now - $_[0],1);
}
sub logComment {
   return common_html::bugzillifyHtml(cgi::htmlQuote($_[0]));
}
sub hexdump {
   return unpack("H*",$_[0]);
}
sub revision {
   my($value,$filenameWithPath,undef) = @_;
   return cgi::Link("$filenameWithPath---checkout.html","rev=$value") . cgi::htmlQuote($value) . "</a>";
}
sub filenameWithIcon {
   my($value,$filenameWithPath,$type) = @_;
   my $out = "";
   if(substr($filenameWithPath,-3) eq "/..") {  # if last three chars are "/.."
      $out .= cgi::Link("$filenameWithPath---ls.html");
      $out .= "<img border=0 SRC=\"/icons/back.gif\" ALT=\"[DIR]\"> Parent Directory";
   } elsif($type eq "dir") {
      $out .= cgi::Link("$filenameWithPath/---ls.html");
      $out .= "<img border=0 SRC=\"/icons/folder.gif\" ALT=\"[DIR]\">";
   } else {
      $out .= cgi::Link("$filenameWithPath---ls.html");
      $out .= "<img border=0 SRC=\"/icons/text.gif\" ALT=\"[TXT]\">";
   };
   $out .= " " . cgi::htmlQuote("$value")."</a></td>\n";
   return $out;
}

my(%colsql,%coltitle,%coladmin,%colfunc,%coldesc,$allcols);
sub DefCol {
   my($nickname,$title,$sql,$colfunc,$descript,$urlparam) = @_;
   $coltitle{$nickname} = $title;  # top of table
   $colsql{$nickname} = $sql;
   $colfunc{$nickname} = $colfunc;   
   $coldesc{$nickname} = $descript; # for column selection
   $allcols .= ".$nickname";
}
#       nick       title             sql                    type                descript                       
DefCol("name",     "name",          "dirent.fname",        \&filenameWithIcon, "Filename");
DefCol("head",     "head",          "dirent.head",         \&revision,         "Revision of head");
DefCol("file_id",  "file_id",       "dirent.file_id",      undef,              "database file id");
DefCol("comment",  "comment",       "dirent.comment",      undef,              "file comment");
DefCol("mode",     "mode",          "dirent.mode",         undef,              "Mode of checked out file");
DefCol("type",     "type",          "dirent.type",         undef,              "File, Dir, Link, Device, etc.");
DefCol("rev",      "rev",           "revision.revision",   \&revision,         "Recent revision (on this branch)");
DefCol("date",     "date",          "revision.timestamp",  \&unixTimestamp,    "When was it committed" );
DefCol("age",      "age",           "revision.timestamp",  \&readableDuration, "Age of recent revision");
DefCol("length",   "length",        "revision.length",     undef,              "File length of displayed revision");
DefCol("author",   "author",        "comment.author",      undef,              "Author of recent revision");
DefCol("hexacl",   "hexacl",        "latest.acl",          \&hexdump,          "Hex of Access Control List") if $config::debug;
DefCol("log",      "last log entry","comment.descr",       \&logComment,       "Full log entry of recent revision");
DefCol("comment_id","comment_id",   "revision.comment_id", undef,              "Log entry of recent revision");
my $default_cols = "name.rev.age.author.log";

sub htmlLsForDir 
{
   my($filename,$file_id,$inforef) = @_;
   if($inforef->{'type'} eq "dir" && $filename !~ /\/$/) {  # if not ending is slash
      $filename .= "/";
   }
   my $cols = $cgi::form{'cols'} || $default_cols;
   my @cols = split(/[^\w\-]+/,$cols); # we don't trust this list of cols
   my @cols2 = ();  # this is were we store the list of cols after we checked they are okay
   my($sql,$heading);
   my $sort = $cgi::form{'sort'} || "name";
   my $desc = ($sort =~ s/^-//) ? " DESC" : "";  # remove leading negation
   $sort = "name" unless(defined $colsql{$sort});
   for my $nick (@cols) {  # for each column we are preparing for
      next if(!defined($colsql{$nick}));
      push(@cols2,$nick);
      $sql .= ",$colsql{$nick}";
      my $color = "000080";
      $color = "05a555" if $sort eq $nick;
      my $direction = "";
      $direction = "-" if $sort eq $nick && ! $desc;
      $heading .= "<th bgcolor=$color>".cgi::Link($filename,"sort=$direction$nick").
                  "<font color=ffffff>$coltitle{$nick}</font></a></th>\n";
   }
   $heading = "<table width=100%><tr>\n$heading</tr>\n";
   my @rows = cvsdb::GetFiles("dirent.fname,dirent.type,revision.state,latest.acl,latest.owner$sql","dirent.parent = $file_id",
                                 $colsql{$sort} . $desc,$cgi::form{'latest'},$cgi::form{'branch'},$cgi::form{'label'});
   my $rowIndex = 0;
   my $table = $heading;
   unshift(@rows,["..","dir"]) unless $filename eq '/';
   my %typecount; 
   my $numRemoveHidden = 0;
   db::getDbNowTime();
   while(my $rowref  = shift(@rows)) {  # for each row
      $table .= ' <tr bgcolor="#' . (($rowIndex & 1) ? "FFFFFF" : "dFdFFF") . "\">\n";
      my($fname,$type,$state,$acl,$owner) = splice(@$rowref,0,5,());
      if((!defined($cgi::form{showRm}) || !$cgi::form{showRm}) && defined($state) && $state eq "dead") {
         $numRemoveHidden++;
         next;
      }
      $typecount{$type}++;
      for my $nick (@cols2) {  # for every column of the row
         my $value = shift(@$rowref) || "";
         $table .= "  <td>";
         if(length $value == 0 && $nick ne "name") {
            $table .= "&nbsp;"; # netscape requires at least one space for background color
         } elsif(defined $colfunc{$nick}) {
            $table .= &{$colfunc{$nick}}($value,"$filename$fname",$type);
         } else {
            $table .= cgi::htmlQuote($value);
         }
         $table .= "</td>\n";   
      }
      $table .= " </tr>\n";
      $table .= "</table>\n$heading" if (($rowIndex++)&127) == 127;  # huge tables display slowly
   } 
   $table .= "</table>\n";
   $cgi::keyword{'direntTable'} = $table;

   my $total = 0;
   $cgi::keyword{'direntTypeCounts'} = join(" + ",grep {$total += $typecount{$_} ; $_ = " $typecount{$_} $_ "} sort(keys(%typecount))).
         " = $total total";
   $cgi::keyword{'showRm'} = $cgi::form{'showRm'} ? "CHECKED" : "";
   $cgi::keyword{'numRemoveHidden'} = $numRemoveHidden;
   #$cgi::keyword{'latestWidget'} = "<input type=text name=latest value=\"" . cgi::htmlQuote($cgi::form{'latest'})."\">";
   $cgi::keyword{'latest'} = cgi::htmlQuote($cgi::form{'latest'});
   $cgi::keyword{'loglink'} = cgi::Link("$filename---log.html","after=".::time2str("%b %e",$db::now - 7*24*3600));
   $cgi::keyword{'allColsLink'} = cgi::Link("$filename---ls.html","cols=$allcols");
   common_html::branchesWidget_keyword("dirent.parent = $file_id"); 

   print cgi::expand_template_keywords("template/ls-dir-template.html");
}

sub ls_html {
   my($filename,$file_id,$inforef) = @_; 
   common_html::content_type("text/html");
   $cgi::keyword{'sortWidget'} = cgi::make_selection_widget("sort",[qw{name age date revision author branches}],
                                      $cgi::form{'sort'} || "name",0,0,0,0,1);
   if($inforef->{'type'} eq 'dir') {
      htmlLsForDir($filename,$file_id,$inforef);
   } else {
      htmlLsForFile($filename,$file_id,$inforef);
   };
}

1;

