#!/usr/bin/perl -w
use strict;
#use diagnostics;

#(This license is from zlib of Jean-loup Gailly and Mark Adler)
#
# Copyright (C) 2002 Frederick Dean <software@fdd.com>
#
#  This software is provided 'as-is', without any express or implied
#  warranty.  In no event will the authors be held liable for any damages
#  arising from the use of this software.
#
#  Permission is granted to anyone to use this software for any purpose,
#  including commercial applications, and to alter it and redistribute it
#  freely, subject to the following restrictions:
#
#  1. The origin of this software must not be misrepresented; you must not
#     claim that you wrote the original software. If you use this software
#     in a product, an acknowledgment in the product documentation would be
#     appreciated but is not required.
#  2. Altered source versions must be plainly marked as such, and must not be
#     misrepresented as being the original software.
#  3. This notice may not be removed or altered from any source distribution.
#
# Contributor(s):    Frederick Dean <software@fdd.com>

package rcsdiff;  # diff format used in RCS (version control) files

use diff qw(diff);

# This routine requires two list references.
# The lists each have one line of file content per list element.
sub rcsdiff
{
   my($f1,$f2) = @_;

   my $diffs = diff($f1, $f2);
   my $answer = "";
   my $growth = 0;  # add to '-' to get '+'
   my($numAdded,$numRemoved) = (0,0);
   while(my $hunk = shift @$diffs) {
      # first process the delete
      my $linechange;
      my $start = undef;
      my $numLines = 0;
      while($linechange = shift @$hunk) {
         my($action,$lnum) = @$linechange;
         last if $action eq '+';
         $start = 1+$lnum if(!defined $start);
         $numLines++;
         $numRemoved++;
      }
      if($numLines) {
         $answer .= "d$start $numLines\n";
      }
      # then process the additions
      $start = undef;
      $numLines = 0;
      my $addtext = "";
      while(defined $linechange) {
         my($action,$lnum,$text) = @$linechange;
         $start = $lnum+$numAdded-$numRemoved if(!defined $start);
         $numLines++;
         $addtext .= $text;
         $linechange = shift @$hunk;
         $numAdded++;
      }
      if($numLines) {
         $answer .= "a$start $numLines\n$addtext";
      }
   } 
   return ($answer,$numAdded,$numRemoved);
};

1;

__END__

From diff.pm....

Here is an example:  The diff of the following two sequences:
 
  a b c e h j l m n p
  b c d e f j k l m r s t
 
Result:
 
 [
   [ [ '-', 0, 'a' ] ],
 
   [ [ '+', 2, 'd' ] ],
 
   [ [ '-', 4, 'h' ] ,
     [ '+', 4, 'f' ] ],
 
   [ [ '+', 6, 'k' ] ],
 
   [ [ '-', 8, 'n' ],
     [ '-', 9, 'p' ],
     [ '+', 9, 'r' ],
     [ '+', 10, 's' ],
     [ '+', 11, 't' ],
   ]
 ]

Assuming each one of the texts included a trailing newline like it must, the 
rcsdiff would look like:

d1 1
a2 1
d
d5 1
a4 1
f
a6 1
k
d9 2
a8 3
r
s
t


#!/usr/bin/perl -w
use strict;
use rcsdiff;
my $a = "a\nb\nc\ne\nh\nj\nl\nm\nn\np\n";
my $b = "b\nc\nd\ne\nf\nj\nk\nl\nm\nr\ns\nt\n";
my @a = split(/^/m,$a);
my @b = split(/^/m,$b);
print rcsdiff::rcsdiff(\@a,\@b);

