#!/usr/bin/perl -w
use strict;

#(This license is from zlib of Jean-loup Gailly and Mark Adler)
#
# Copyright (C) 2002 Frederick Dean <software@fdd.com>
#
#  This software is provided 'as-is', without any express or implied
#  warranty.  In no event will the authors be held liable for any damages
#  arising from the use of this software.
#
#  Permission is granted to anyone to use this software for any purpose,
#  including commercial applications, and to alter it and redistribute it
#  freely, subject to the following restrictions:
#
#  1. The origin of this software must not be misrepresented; you must not
#     claim that you wrote the original software. If you use this software
#     in a product, an acknowledgment in the product documentation would be
#     appreciated but is not required.
#  2. Altered source versions must be plainly marked as such, and must not be
#     misrepresented as being the original software.
#  3. This notice may not be removed or altered from any source distribution.
#
# Contributor(s):    Frederick Dean <software@fdd.com>

BEGIN { push @INC, "/var/www/html/dcvs" };

#
#  This script will read in a unified-context-diff (created with diff -u)
#  and create a data structure like those generated by diff::diff()
#  We then use our merge to try to apply it to the target.
#  Thus, we can compare ourselves to GNU patch.
#

use merge;

if($#ARGV != 1) {   # if not two command line args
   print("usage: mergetest patchfile target\n");
   exit -1;
}
my($patchFilename,$targetfilename) = @ARGV;

# Load the patch, and create the diff datastructure.
# The "hunk" of the context diff might encompass several "hunks" of the diff::diff() output.
open(PATCH,"<$patchFilename") or die("Could not open $patchFilename\n$!");
my @diff;  # This is a structure like diff::diff() creates.  See __END__
my(@orig1,@orig2);  # As far as we can tell from the patch these are original files.  One line per element.
my $numCommonLines = 0;  
while(defined(my $line = <PATCH>)) {   # for each hunk delimited with @@
   next unless $line =~ /^\@\@/;  # skip until a line begins with @@
   if($line =~ /^diff/ && $#orig1 != -1) {  # if we see the start of a file, and we already have seen a hunk
      print(STDERR "only first file in patch used\n"); 
      last;
   }
   $line =~ /^\@\@\s+-(\d+),(\d+)\s+\+(\d+),(\d+)\s+\@\@/ or die("not unified context diff?");
   my($offset1,$numLinesThisHunk1,$offset2,$numLinesThisHunk2) = ($1,$2,$3,$4);
   my $numMissingContextLines = ($offset1 - $#orig1 -2);
   die unless $numMissingContextLines == ($offset2 - $#orig2 -2);
   push(@orig1, ("### missing context\n") x $numMissingContextLines);
   push(@orig2, ("### missing context\n") x $numMissingContextLines);
   $numCommonLines += $numMissingContextLines;
   my @hunk;  # A diff::diff() hunk being built.
   while(defined($line = <PATCH>)) {  # for each line of this hunk
      $line =~ s/^(.)// or die("empty line not expected"); # remove first character
      if($1 eq "-") {
         push(@orig1,$line);
         $numLinesThisHunk1--;
         push(@hunk ,[ '-', $numCommonLines, $line ]);
      } elsif($1 eq "+") {
         push(@orig2,$line);
         $numLinesThisHunk2--;
         push(@hunk ,[ '+', $numCommonLines, $line ]);
      } elsif($1 eq " ") {
         push(@orig1,$line);
         push(@orig2,$line);
         $numLinesThisHunk1--;
         $numLinesThisHunk2--;
         push @diff, \@hunk;
         @hunk = ();
         $numCommonLines++;
      } else {
         die("unexpected prefix character");
      };
      last if $numLinesThisHunk1 == 0 && $numLinesThisHunk2 == 0;
      die if $numLinesThisHunk1 < 0 || $numLinesThisHunk2 < 0;
   }
   push @diff, \@hunk;
   @hunk = ();
}
close(PATCH);

open(TARGET,"<$targetFilename") or die("could not open $targetFilename\n$!");
read(TARGET,my $target,1000000);
close(TARGET);

my($answer,$conflict) = merge::merge(join("",@orig1),$target,"1.2",join("",@orig2),"patched");
print $answer;

__END__


===================================================================
RCS file: /home/cvs/dcvs/test/db_errors_dirent.pl,v
retrieving revision 1.1
diff -u -r1.1 db_errors_dirent.pl
--- db_errors_dirent.pl 17 Mar 2002 16:48:42 -0000      1.1
+++ db_errors_dirent.pl 3 Apr 2002 01:52:04 -0000
@@ -26,8 +26,21 @@
 use db;
 #use cvsdb;

+if(! defined $ARGV[0] || $ARGV[0] ne "--dataloss") {
+   print STDERR "\n".
+                "      Your DCVS database will loose all revisions!\n".
+                "      Only run this script if you want to loose data!\n".
+                "      Use the --dataloss flag if you are sure.\n".
+                "      The database was not altered.\n".
+                "\n";
+   exit -1;
+}


         a b c e h j l m n p
         b c d e f j k l m r s t

       Result:

        [
          [ [ '-', 0, 'a' ] ],

          [ [ '+', 2, 'd' ] ],

          [ [ '-', 4, 'h' ] ,
            [ '+', 4, 'f' ] ],

          [ [ '+', 6, 'k' ] ],

          [ [ '-', 8, 'n' ],
            [ '-', 9, 'p' ],
            [ '+', 9, 'r' ],
            [ '+', 10, 's' ],
            [ '+', 11, 't' ],
          ]
        ]



